package cn.fastfun.service.impl;

import cn.fastfun.controller.param.*;
import cn.fastfun.service.AppDeviceLogService;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.AppDeviceLog;
import cn.fastfun.service.entity.AppImeiHistory;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.QueryParam;
import com.bridge.dto.QueryParamExp;
import com.bridge.exception.ApiRuntimeException;
import com.bridge.service.JpaService;
import com.bridge.service.impl.JpaServiceImp;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.persistence.Query;
import javax.transaction.Transactional;
import java.math.BigInteger;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;

@Slf4j
@Service
public class AppDeviceLogServiceImpl extends JpaServiceImp<AppDeviceLog, String> implements AppDeviceLogService {

    @Resource
    AppDeviceService appDeviceService;

    @Resource
    AppDeviceLogService appDeviceLogService;

    @Resource
    UtilService utilService;
    //业务类
    @Resource(name = "appImeiHistoryService")
    JpaService<AppImeiHistory, String> appImeiHistoryService;

    @Override
    @Transactional
    public void putInStorage(LibraryInFormParam param) {
        List<AppDevice> deviceList = appDeviceService.findAll(QueryParamExp.in("sn", param.getSn().toArray(new String[]{})));
        param.setSn(new ArrayList<>());
        deviceList.forEach(p -> param.getSn().add(p.getSn()));
        if (!CollectionUtils.isEmpty(param.getSn())) {
            // 查询当前操作id的最大值
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device_log where operate_id is not null");
            log.info("SQL: {}", sql.toString());
            ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
            List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
            BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
            operateID = operateID.add(BigInteger.valueOf(1));
            Date time = new Date();

            BigInteger finalOperateID = operateID;

            param.getSn().forEach(p -> {
                String batchNum;
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
                batchNum = appDevice.getBatchNum();
                if (appDevice.getStatus() < 1) {
                    appDevice.setStatus(1); // 已入库
                    appDevice.setInstorageTime(time);
                    appDeviceService.save(appDevice);

                    AppDeviceLog appDeviceLog = new AppDeviceLog(p, appDevice.getImei()).toInStorage(time, batchNum, finalOperateID);
                    appDeviceLog.setOperator(utilService.getUserName());
                    save(appDeviceLog); // 记录入库日志
                }
            });
        }
    }

    @Override
    @Transactional
    public void transfer(TransferFormParam param) {
        if (CollectionUtils.isEmpty(param.getSn()) && !CollectionUtils.isEmpty(param.getBatchNum())) {
            List<AppDevice> deviceList = appDeviceService.findAll(QueryParamExp.in("batchNum", param.getBatchNum().toArray(new String[]{})));
            param.setSn(new ArrayList<>());
            deviceList.forEach(p -> param.getSn().add(p.getSn()));
        }

        if (!CollectionUtils.isEmpty(param.getSn())) {
            // 查询当前操作id的最大值
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device_log where operate_id is not null");
            log.info("SQL: {}", sql.toString());
            ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
            List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
            BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
            operateID = operateID.add(BigInteger.valueOf(1));
            Date time = new Date();

            BigInteger finalOperateID = operateID;
            AtomicBoolean allowTransfer = new AtomicBoolean(true);
            //批量调拨时，若有不符合调拨条件的电池，则停止本次批量调拨

            param.getSn().forEach(p -> {
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (null == appDevice) {
                    allowTransfer.set(false);
                    throw new ApiRuntimeException(p + "设备信息不存在!");
                }
                if (1 != appDevice.getStatus() && 2 != appDevice.getStatus() && 3 != appDevice.getStatus()) {
                    allowTransfer.set(false);
                    throw new ApiRuntimeException(p + "不是入库或出库状态，不能调拨!");
                }
                if (1 != appDevice.getCheckStatus()) {
                    allowTransfer.set(false);
                    throw new ApiRuntimeException(p + "自检未通过，不能调拨!");
                }
            });
            param.getSn().forEach(p -> {
                String batchNum;
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                batchNum = appDevice.getBatchNum();
                if (2 == appDevice.getStatus()) {
                    return;
                }
                appDevice.setStatus(2); // 已划拨
                appDevice.setTransferTime(time);
                appDevice.setOwnerId(param.getCustomId()); // 设置归属
                appDeviceService.save(appDevice);

                AppDeviceLog appDeviceLog = new AppDeviceLog(p, appDevice.getImei()).toTransfer(param, time, batchNum, finalOperateID);
                appDeviceLog.setOperator(utilService.getUserName());
                save(appDeviceLog); // 记录划拨日志
            });
        }
    }

    @Override
    public void transferBack(TransferFormParam param) {
        AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", param.getSn().get(0)));
        // 查询当前操作id的最大值
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device_log where operate_id is not null");
        log.info("SQL: {}", sql.toString());
        ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
        List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
        BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
        operateID = operateID.add(BigInteger.valueOf(1));
        Date time = new Date();

        BigInteger finalOperateID = operateID;
        String batchNum;
        if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
        if (3 != appDevice.getStatus()) throw new ApiRuntimeException("设备不是出库状态!");
        batchNum = appDevice.getBatchNum();
        appDevice.setInstorageTime(time);
        appDevice.setStatus(1); // 已入库
        appDevice.setTransferBackTime(time);
        appDeviceService.save(appDevice);

        AppDeviceLog appDeviceLog = new AppDeviceLog(appDevice.getSn(), appDevice.getImei()).transferBack(param, time, batchNum, finalOperateID);
        appDeviceLog.setOperator(utilService.getUserName());
        save(appDeviceLog); // 记录调回日志
    }

    @Override
    @Transactional
    public void outStorage(LibraryOutFormParam param) {
        if (CollectionUtils.isEmpty(param.getSn()) && !CollectionUtils.isEmpty(param.getBatchNum())) {
            List<AppDevice> deviceList = appDeviceService.findAll(QueryParamExp.in("batchNum", param.getBatchNum().toArray(new String[]{})));
            param.setSn(new ArrayList<>());
            deviceList.forEach(p -> param.getSn().add(p.getSn()));
        }

        if (!CollectionUtils.isEmpty(param.getSn())) {
            // 查询当前操作id的最大值
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device_log where operate_id is not null");
            log.info("SQL: {}", sql.toString());
            ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
            List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
            BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
            operateID = operateID.add(BigInteger.valueOf(1));
            Date time = new Date();

            //批量出库时，若有不符合出库条件的电池，则停止本次批量出库
            AtomicBoolean allowTransfer = new AtomicBoolean(true);
            param.getSn().forEach(p -> {
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (null == appDevice) {
                    allowTransfer.set(false);
                    throw new ApiRuntimeException(p + "设备信息不存在!");
                }
                if ( 2 != appDevice.getStatus() && 3 != appDevice.getStatus()) {
                    allowTransfer.set(false);
                    throw new ApiRuntimeException(p + "不是调拨状态，不能出库!");
                }
            });
            BigInteger finalOperateID = operateID;
            param.getSn().forEach(p -> {
                String batchNum;
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (3 == appDevice.getStatus()) {
                    return;
                }
                batchNum = appDevice.getBatchNum();
                appDevice.setOutstorageTime(time);
                appDevice.setStatus(3); // 已出库
                appDeviceService.save(appDevice);
                AppDeviceLog appDeviceLog = new AppDeviceLog(p, appDevice.getImei()).outStorage(param, time, batchNum, finalOperateID);
                appDeviceLog.setOperator(utilService.getUserName());
                save(appDeviceLog); // 记录出库日志
            });
        }
    }

    @Override
    @Transactional
    public void handle(HandelFormParam param) {
        // 查询当前操作id的最大值
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device_log where operate_id is not null");
        log.info("SQL: {}", sql.toString());
        ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
        List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
        BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
        operateID = operateID.add(BigInteger.valueOf(1));
        Date time = new Date();

        BigInteger finalOperateID = operateID;
        param.getSn().forEach(p -> {
            String batchNum;
            AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
            if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
            if (1 != appDevice.getStatus()) throw new ApiRuntimeException("设备不是入库状态!");
            batchNum = appDevice.getBatchNum();
            appDevice.setHandleTime(time);
            appDevice.setStatus(4); // 处置
            appDeviceService.save(appDevice);

            AppDeviceLog appDeviceLog = new AppDeviceLog(p, appDevice.getImei()).toHandle(param, time, batchNum, finalOperateID);
            appDeviceLog.setOperator(utilService.getUserName());
            save(appDeviceLog); // 记录处置日志
        });
    }
}
